//-------------------------------------------------------------------------------
//      UART_LCD Ver1.00
//
//
//                                              Software design    : Masaru Osuga
//                                                         DATE    :  2009/03/22 
//
// Reorganization history--------------------------------------------------------
// Ver1.00		2009/03/22		New Software release.　　　　　            　　　　　　　
// Ver1.01		2009/03/26		・カーソルのオン・オフ切換を追加
//								・受信割り込みを使用しレスポンスの向上　　ほか　　　　　　　　　　　　　
//-------------------------------------------------------------------------------


#include <m8c.h>        // part specific constants and macros
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules
#include <string.h>
#include <stdio.h>

#pragma jis
#define NULL 0x00

#define DEBUG 0			//デバッグ用　0:非表示　1:表示
#define X_MAX 2			//ＬＣＤの最大行
#define Y_MAX 8			//ＬＣＤの最大桁

BOOL fCursor ;
BYTE Contrast;			// LCD Contrast
BYTE Position_X=0,Position_Y=0;
char     * temp;

void interrupt INT_UART_RX(void) @ 0x28 
{
  char recv_data;
  recv_data = UART_cGetChar(); // read UART
  temp = recv_data;
 }

void wait(long l){
	while(l--);			//long MAX	 2147483647L
	}
void pos(BYTE x,BYTE y){
	LCD_Position(x,y);
}
void putch( char c ){
	BYTE str[] = " ";
	strncpy(str,&c,1);
	LCD_PrString(str);
}
void LCD_CLS(void){
	pos(0,0); for(BYTE i=0;i<Y_MAX;i++)printf(" ");
	pos(1,0); for(BYTE i=0;i<Y_MAX;i++)printf(" ");
}
void SetLCD_Contrast(char cont){
	if(cont == 0xff)cont=0;
	if(cont>31)cont =31;
	Contrast = cont;
	Vreg_cr0 = 0b10100000 + Contrast;
}


void main()
{
	//char     * temp;
    fCursor = TRUE;
    UART_Start(UART_PARITY_NONE);		// Enable UART
    UART_CPutString("PSoC UART LCD  Ver1.01\r\n");
    	
	Vreg_Start(3);						// ＬＣＤ用　Ｖｏ制御
    LCD_Start();
	LCD_CLS();
	SetLCD_Contrast(20);
	pos(0,0); printf("  PSoC  ");
	pos(1,0); printf("UART LCD");
	wait(1000000L);
	for(BYTE i = 0 ; i < 10 ; i++ ){
		SetLCD_Contrast( Contrast-- );
		wait(100000L); }
	LCD_CLS();
	SetLCD_Contrast(20);wait(500000L);
	UART_CPutString("Ready\r\n");
    UART_IntCntl(UART_ENABLE_RX_INT );
	M8C_EnableGInt;						// Enable Global interrupts
	
	while(1){
		
	    //temp = UART_cGetChar();		//ＵＡＲＴより１文字を取得
		while(!temp);					//
										//ＬＣＤ制御
		if((BYTE)temp == 0x01){			//コントラストＵＰ      [CTRL+A]
			SetLCD_Contrast(++Contrast);
			#if ( DEBUG )
				UART_CPutString("\r\n[+1]Contrast:");UART_PutSHexByte(Contrast);
			#endif
			temp = NULL;				//データリセット
		}
		if((BYTE)temp == 0x02){			//コントラストＤＯＷＮ  [CTRL+B]
			SetLCD_Contrast(--Contrast);
			#if ( DEBUG )
				UART_CPutString("\r\n[-1]Contrast:");UART_PutSHexByte(Contrast);
			#endif
			temp = NULL;				//データリセット
		}
		if((BYTE)temp == 0x03){			//ＬＣＤクリア　　　　　[CTRL+C]
			LCD_CLS();
			Position_X=(X_MAX);
			Position_Y=(Y_MAX);
			temp = NULL;				//データリセット
		}
		if((BYTE)temp == 0x04){			//カーソルのオン・オフ切換
			if(fCursor){
				fCursor = FALSE;
				LCD_Control(LCD_CURSOR_WINK);
				UART_CPutString("CursorON\r\n");
			} else {
				fCursor = TRUE;
				LCD_Control(LCD_CURSOR_OFF  );
				UART_CPutString("CursorOFF\r\n");
			}
			temp = NULL;
		}
		if((BYTE)temp == 0x0d){			//改行　　　　　　　　　[RETURN]
			if(Position_Y == Y_MAX ){
			#if ( DEBUG )
				UART_PutChar('+');
			#endif
				pos(Position_X,0); for(BYTE i=0;i<Y_MAX;i++)printf(" ");
				Position_Y = Y_MAX;
			}
			else {
				pos(Position_X,Position_Y); for(BYTE i = Position_Y;i<Y_MAX;i++)printf(" ");
				Position_Y = Y_MAX;
			}
			UART_PutCRLF();
			temp = NULL;				//データリセット
		}
		if((BYTE)temp  < 0x20){
			temp = NULL;	//データをASCIIデータ以外を除外
		}
		
		if(temp){						//データがあれば以下を実行
			while(UART_bReadTxStatus()== UART_TX_BUFFER_EMPTY );//ＵＡＲＴ　送信バッファが空か？
			UART_SendData(temp);		//ＵＡＲＴに表示データを送る
			pos(Position_X,Position_Y);	
			LCD_WriteData((BYTE)temp);	//ＬＣＤ　に表示データを送る
			
		#if ( DEBUG )
			UART_PutChar('(');UART_PutSHexByte(temp);UART_PutChar(')');//取得文字情報
			UART_PutChar('[');UART_PutSHexByte(Position_X);UART_PutChar(',');UART_PutSHexByte(Position_Y);UART_PutChar(']');
			//表示位置情報
		#endif
			temp = NULL;				//データリセット
		}
										//表示位置の更新
		if((++Position_Y)>=(Y_MAX)){	//桁がLCDの枠外？
			Position_X++  ; Position_Y = 0; UART_PutCRLF(); 
		}
		if(( Position_X )>(X_MAX-1)){	//行がLCDの枠外？
			Position_X = 0; 
		}
	}
}										//main loop end.
